package org.apache.bcel.generic;

import java.util.StringTokenizer;
import org.apache.bcel.Const;
import org.apache.bcel.classfile.Constant;
import org.apache.bcel.classfile.ConstantCP;
import org.apache.bcel.classfile.ConstantPool;
import org.apache.bcel.classfile.Utility;

public abstract class InvokeInstruction extends FieldOrMethod implements ExceptionThrower, StackConsumer, StackProducer {
   InvokeInstruction() {
   }

   protected InvokeInstruction(short opcode, int index) {
      super(opcode, index);
   }

   public int consumeStack(ConstantPoolGen cpg) {
      int sum;
      if (super.getOpcode() != 184 && super.getOpcode() != 186) {
         sum = 1;
      } else {
         sum = 0;
      }

      String signature = this.getSignature(cpg);
      sum += Type.getArgumentTypesSize(signature);
      return sum;
   }

   public Type[] getArgumentTypes(ConstantPoolGen cpg) {
      return Type.getArgumentTypes(this.getSignature(cpg));
   }

   public String getClassName(ConstantPoolGen cpg) {
      ConstantPool cp = cpg.getConstantPool();
      ConstantCP cmr = (ConstantCP)cp.getConstant(super.getIndex());
      String className = cp.getConstantString(cmr.getClassIndex(), (byte)7);
      return Utility.pathToPackage(className);
   }

   public String getMethodName(ConstantPoolGen cpg) {
      return this.getName(cpg);
   }

   public Type getReturnType(ConstantPoolGen cpg) {
      return Type.getReturnType(this.getSignature(cpg));
   }

   public Type getType(ConstantPoolGen cpg) {
      return this.getReturnType(cpg);
   }

   public int produceStack(ConstantPoolGen cpg) {
      String signature = this.getSignature(cpg);
      return Type.getReturnTypeSize(signature);
   }

   public String toString(ConstantPool cp) {
      Constant c = cp.getConstant(super.getIndex());
      StringTokenizer tok = new StringTokenizer(cp.constantToString(c));
      String opcodeName = Const.getOpcodeName(super.getOpcode());
      StringBuilder sb = new StringBuilder(opcodeName);
      if (tok.hasMoreTokens()) {
         sb.append(" ");
         sb.append(tok.nextToken().replace('.', '/'));
         if (tok.hasMoreTokens()) {
            sb.append(tok.nextToken());
         }
      }

      return sb.toString();
   }
}
